#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const entry_1 = require("./lib/entry");
const logger_flx_1 = require("logger-flx");
const starter_1 = require("./lib/starter");
const connector_1 = require("./lib/connector");
const logger = new logger_flx_1.Logger({
    mode: entry_1.default.logs,
    type: true,
    timestamp: "none",
    enable: true
});
const connector_config = {
    update: entry_1.default.update,
    target: entry_1.default.target,
    interval: entry_1.default.interval,
    url: entry_1.default.url,
    tmp: entry_1.default.tmp,
    keys: entry_1.default.keys
};
logger.log(chalk.bold.white.bgGray("hcs-client config:"), "debug");
for (const key in entry_1.default) {
    logger.log(chalk.bold.white.bgGray(`  ${key} = ${entry_1.default[key]}`), "debug");
}
const bootstrap = async () => {
    const connector = new connector_1.Connector(connector_config, logger);
    await connector.run();
    if (entry_1.default.exec !== undefined) {
        if (connector.heathy === false) {
            if (entry_1.default.critical === true) {
                logger.error("[HCL-Client] Connector is unhealthy. Shutdown critical process.");
                process.exit(1);
            }
            else {
                logger.warn("[HCL-Client] Connector is unhealthy. Process not critical.");
            }
        }
        const starter_config = {
            exec: entry_1.default.exec,
            cwd: entry_1.default.cwd,
            restart_interval: entry_1.default.restart_interval,
            webhook: entry_1.default.webhook,
            shell: entry_1.default.shell
        };
        const starter = new starter_1.Starter(starter_config, logger);
        starter.run();
        connector.on("change", () => {
            starter.restart();
        });
        starter.on("close", () => {
            process.exit();
        });
        starter.on("error", () => {
            process.exit(1);
        });
        process.on("SIGTERM", () => {
            logger.log("Termination signal received");
            starter.stop();
            connector.stop();
        });
    }
};
bootstrap();
