"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpSource = void 0;
const node_fetch_1 = require("node-fetch");
const chalk = require("chalk");
class HttpSource {
    constructor(_url_server, _logger) {
        this._url_server = _url_server;
        this._logger = _logger;
        this._url_server = this._url_server.replace(/\/$/gi, "");
    }
    getList(url_file) {
        return new Promise((resolve, reject) => {
            const url = `${this._url_server}/v1/store/list/${url_file}`;
            this._logger.log(`[HCL-Client] Request: ${chalk.grey(url)}`, "dev");
            node_fetch_1.default(url).then((response) => {
                if (response.ok === false) {
                    return reject(new Error(`Http request return ${response.status}. ${response.statusText}`));
                }
                return response.json();
            }).then((body) => {
                if (body.status === "success") {
                    return resolve(body.data);
                }
                return reject(new Error(`Request getList for path ${url_file} return status ${body.status}`));
            }).catch((error) => {
                reject(error);
            });
        });
    }
    getFile(url_file) {
        return new Promise((resolve, reject) => {
            const url = `${this._url_server}/v1/store/get/${url_file}`;
            this._logger.log(`[HCL-Client] Request: ${chalk.grey(url)}`, "dev");
            node_fetch_1.default(url).then((response) => {
                if (response.ok === false) {
                    return reject(new Error(`Http request return ${response.status}. ${response.statusText}`));
                }
                return response.json();
            }).then((body) => {
                if (body.status === "success") {
                    return resolve(body.data);
                }
                return reject(new Error(`Request getFile for path "${url_file}" return status ${body.status}`));
            }).catch((error) => {
                reject(error);
            });
        });
    }
    getHashes(url_files) {
        return new Promise((resolve, reject) => {
            const url = `${this._url_server}/v1/store/hashes`;
            this._logger.log(`[HCL-Client] Request: ${chalk.grey(url)}`, "dev");
            node_fetch_1.default(url, {
                method: "post",
                body: JSON.stringify(url_files),
                headers: {
                    "Content-Type": "application/json"
                }
            }).then((response) => {
                if (response.ok === false) {
                    return reject(new Error(`Http request return ${response.status}. ${response.statusText}`));
                }
                return response.json();
            }).then((body) => {
                if (body.status === "success") {
                    return resolve(body.data);
                }
                return reject(new Error(`Request getHashes return status "${body.status}"`));
            }).catch((error) => {
                reject(error);
            });
        });
    }
}
exports.HttpSource = HttpSource;
