"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const chalk = require("chalk");
const fs = require("fs");
const path = require("path");
const finder = require("find-package-json");
const pkg = finder(__dirname).next().value;
commander_1.program.version(`version: ${pkg.version}`, "-v, --version", "output the current version.");
commander_1.program.name(pkg.name);
commander_1.program.requiredOption("-u, --url <type>", "Url to config server (Environment variable: HCS_CLIENT_URL=<type>). Example: --webhook http://config-server:5000/api");
commander_1.program.option("-e, --exec <type>", "Start command application (Environment variable: HCS_CLIENT_EXEC=<type>). Example: node ./app.js");
commander_1.program.option("-w, --webhook <type>", "Url to webhook application (Environment variable: HCS_CLIENT_WEBHOOK=<type>). Example: --webhook http://myapp:5000/webhook");
commander_1.program.option("-i, --interval <number>", "Interval checking files/folders in seconds (Environment variable: HCS_CLIENT_INTERVAL=<number>). Example: --interval 10", "10");
commander_1.program.option("-ri, --restart_interval <number>", "Restart interval app (Environment variable: HCS_CLIENT_RESTART_INTERVAL=<number>). Example: --restart_interval 2", "2");
commander_1.program.option("-t, --target [letters...]", "Watching string/array of string files path (Environment variable: HCS_CLIENT_TARGET=<string[]>). Example: --target /configs/app_config.json:/configs/app_config.json");
commander_1.program.option("-c, --cwd <type>", "Path to workdir (Environment variable: HCS_CLIENT_CWD=<type>). Example: --cwd /my_cwd", `${process.cwd()}`);
commander_1.program.option("-tm, --tmp <type>", "Path to tmp dir (Environment variable: HCS_CLIENT_TMP=<type>). Example: --tmp /my_tmp", "tmp_hcs");
commander_1.program.option("-up, --update", "Flag for watch targets update (Environment variable: HCS_CLIENT_UPDATE=(true|false)).", false);
commander_1.program.option("-cr, --critical", "Flag for critical process (Environment variable: HCS_CLIENT_CRITICAL=(true|false)).", false);
commander_1.program.option("-l, --logs <type>", "Logs details, can be prod, dev or debug (Environment variable: HCS_CLIENT_LOGS=<type>). Example: --logs prod", "prod");
commander_1.program.option("-k, --keys [letters...]", "String/array path to keys file. (Environment variable: HCS_CLIENT_KEYS=<string[]>). Example: --keys /keys.json /keys.toml");
commander_1.program.option("-s, --shell", "Flag shell mode (Environment variable: HCS_CLIENT_SHELL=(true|false)).", false);
commander_1.program.parse(process.argv);
const config = {
    url: commander_1.program.url,
    webhook: commander_1.program.webhook,
    interval: parseInt(commander_1.program.interval),
    restart_interval: parseInt(commander_1.program.restart_interval),
    exec: commander_1.program.exec,
    target: commander_1.program.target,
    cwd: commander_1.program.cwd,
    update: commander_1.program.update,
    shell: commander_1.program.shell,
    logs: commander_1.program.logs.toLowerCase(),
    keys: commander_1.program.keys,
    critical: commander_1.program.critical,
    tmp: commander_1.program.tmp
};
if (process.env["HCS_CLIENT_WEBHOOK"] !== undefined) {
    config.webhook = process.env["HCS_CLIENT_WEBHOOK"].trim();
}
if (process.env["HCS_CLIENT_URL"] !== undefined) {
    config.url = process.env["HCS_CLIENT_URL"].trim();
}
if (process.env["HCS_CLIENT_INTERVAL"] !== undefined) {
    config.interval = parseInt(process.env["HCS_CLIENT_INTERVAL"].trim());
}
if (process.env["HCS_CLIENT_RESTART_INTERVAL"] !== undefined) {
    config.restart_interval = parseInt(process.env["HCS_CLIENT_RESTART_INTERVAL"].trim());
}
if (process.env["HCS_CLIENT_EXEC"] !== undefined) {
    config.exec = process.env["HCS_CLIENT_EXEC"].trim();
}
if (process.env["HCS_CLIENT_TARGET"] !== undefined) {
    config.target = JSON.parse(process.env["HCS_CLIENT_TARGET"].trim());
}
if (process.env["HCS_CLIENT_LOGS"] !== undefined) {
    config.logs = process.env["HCS_CLIENT_LOGS"].trim().toLowerCase();
}
if (process.env["HCS_CLIENT_KEYS"] !== undefined) {
    config.keys = JSON.parse(process.env["HCS_CLIENT_KEYS"].trim());
}
if (process.env["HCS_CLIENT_UPDATE"] !== undefined) {
    if (process.env["HCS_CLIENT_UPDATE"].trim().toLowerCase() === "true") {
        config.update = true;
    }
    else {
        config.update = false;
    }
}
if (process.env["HCS_CLIENT_SHELL"] !== undefined) {
    if (process.env["HCS_CLIENT_SHELL"].trim().toLowerCase() === "true") {
        config.shell = true;
    }
    else {
        config.shell = false;
    }
}
if (process.env["HCS_CLIENT_CWD"] !== undefined) {
    config.cwd = process.env["HCS_CLIENT_CWD"].trim();
}
if (process.env["HCS_CLIENT_TMP"] !== undefined) {
    config.tmp = process.env["HCS_CLIENT_TMP"].trim();
}
if (process.env["HCS_CLIENT_CRITICAL"] !== undefined) {
    if (process.env["HCS_CLIENT_CRITICAL"].trim().toLowerCase() === "true") {
        config.critical = true;
    }
    else {
        config.critical = false;
    }
}
if (config.webhook !== undefined) {
    config.webhook = config.webhook.trim();
    if (!/^http(s|)\:\/\/(.*\:.*@|)[a-z0-9]{1}[-a-z0-9.]{0,128}(\:[0-9]{1,5}|)(\/|)(.*|)$/gi.test(config.webhook)) {
        console.error(chalk.red("[ERROR] Webhook key not correspond regexp /^http(s|)\\:\\/\\/(.*\\:.*@|)[a-z0-9]{1}[-a-z0-9.]{0,128}(\\:[0-9]{1,5}|)(\\/|)(.*|)$/gi"));
        process.exit(1);
    }
}
if (config.url !== undefined) {
    config.url = config.url.trim();
    if (!/^http(s|)\:\/\/(.*\:.*@|)[a-z0-9]{1}[-a-z0-9.]{0,128}(\:[0-9]{1,5}|)(\/|)(.*|)$/gi.test(config.url)) {
        console.error(chalk.red("[ERROR] Webhook key not correspond regexp /^http(s|)\\:\\/\\/(.*\\:.*@|)[a-z0-9]{1}[-a-z0-9.]{0,128}(\\:[0-9]{1,5}|)(\\/|)(.*|)$/gi"));
        process.exit(1);
    }
}
if (config.exec !== undefined) {
    config.exec = config.exec.trim();
    config.cwd = path.resolve(process.cwd(), config.cwd.trim());
    if (config.exec.length <= 0) {
        console.error(chalk.red("[ERROR] Exec key is empty"));
        process.exit(1);
    }
    if (config.cwd.length <= 0) {
        console.error(chalk.red("[ERROR] Cwd key is empty"));
        process.exit(1);
    }
    if (!fs.existsSync(config.cwd)) {
        console.error(chalk.red(`[ERROR] Cwd folder ${config.cwd} not found`));
        process.exit(1);
    }
    if (config.interval <= 0) {
        console.error(chalk.red("[ERROR] Interval key must be more than 0"));
        process.exit(1);
    }
}
if (!Array.isArray(config.keys)) {
    config.keys = [];
}
if (config.logs !== "prod" && config.logs !== "dev" && config.logs !== "debug") {
    console.error(chalk.red("[ERROR] Logs must be prod, dev or debug"));
    process.exit(1);
}
if (Array.isArray(config.target)) {
    for (let item of config.target) {
        item = item.trim();
        if (!/^.*\:.*$/gi.test(item)) {
            console.error(chalk.red("[ERROR] Target item not correspond regexp /^.*\\:.*$/gi"));
            process.exit(1);
        }
    }
}
else {
    config.target = [];
}
exports.default = config;
