"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Starter = void 0;
const fs = require("fs");
const path = require("path");
const events_1 = require("events");
const child_process_1 = require("child_process");
const node_fetch_1 = require("node-fetch");
const chalk = require("chalk");
const fkill = require("fkill");
__exportStar(require("./interfaces"), exports);
class Starter extends events_1.EventEmitter {
    constructor(_config, _logger) {
        super();
        this._config = _config;
        this._logger = _logger;
        this._full_cwd_path = path.resolve(process.cwd(), this._config.cwd);
        this._stopping_flag = false;
        this._restarting_flag = false;
        this._closed_flag = true;
        if (!fs.existsSync(this._full_cwd_path)) {
            fs.mkdirSync(this._full_cwd_path, {
                recursive: true
            });
        }
    }
    run() {
        if (this._closed_flag === false) {
            return;
        }
        this._closed_flag = false;
        this._restarting_flag = false;
        this._logger.log("[HCL-Client] App starting ...");
        this._logger.log(`[HCL-Client] Spawn command ${chalk.cyan(this._config.exec)}, workdir ${chalk.cyan(this._full_cwd_path)}`);
        let executer;
        let args = [];
        if (this._config.shell === true) {
            executer = this._config.exec.trim();
        }
        else {
            args = this._config.exec.trim().split(" ");
            executer = args[0];
            args.splice(0, 1);
        }
        this._app = child_process_1.spawn(executer, args, {
            cwd: this._full_cwd_path,
            env: process.env,
            stdio: ["inherit", "inherit", "inherit"],
            shell: this._config.shell
        });
        this._app.on("exit", (code) => {
            this._closed_flag = true;
            this._logger.log(`[HCL-Client] App closed, with code ${chalk.cyan(code)}`, "dev");
            if (this._stopping_flag === true) {
                return;
            }
            if (this._restarting_flag === false) {
                this.emit("close");
                return;
            }
            this._id_interval = setTimeout(() => {
                this.run();
            }, this._config.restart_interval * 1000);
            this._logger.log(`[HCL-Client] App restart after ${chalk.gray(this._config.restart_interval)} sec`);
        });
        this._app.on("error", (error) => {
            this._logger.error("[HCL-Client] Starting app error");
            this._logger.log(error);
            this.emit("error");
        });
        this._logger.log(`[HCL-Client] App started pid: ${chalk.cyan(this._app.pid)}`, "dev");
    }
    stop() {
        if (this._closed_flag === true) {
            return;
        }
        this._logger.log("[HCL-Client] Stopping app ...");
        this._closed_flag = true;
        this._stopping_flag = true;
        this._restarting_flag = false;
        clearTimeout(this._id_interval);
        this._logger.log(`[HCL-Client] Kill process pid ${chalk.cyan(this._app.pid)}`, "debug");
        if (this._config.shell === true) {
            fkill(this._app.pid, {
                force: true,
                tree: true
            });
        }
        else {
            this._app.kill("SIGTERM");
        }
    }
    restart() {
        if (this._restarting_flag === true || this._closed_flag === true || this._stopping_flag === true) {
            return;
        }
        this._logger.log("[HCL-Client] Restarting app ...");
        this._restarting_flag = true;
        clearTimeout(this._id_interval);
        if (this._config.webhook !== undefined) {
            node_fetch_1.default(this._config.webhook).catch((error) => {
                this._logger.log(`[HCL-Client] Error webhook request: ${error.message}`);
            });
        }
        else {
            this._logger.log(`[HCL-Client] Kill process pid ${chalk.cyan(this._app.pid)}`, "debug");
            if (this._config.shell === true) {
                fkill(this._app.pid, {
                    force: true,
                    tree: true
                });
            }
            else {
                this._app.kill("SIGTERM");
            }
        }
    }
    get close() {
        if (this._restarting_flag === true) {
            return false;
        }
        return this._closed_flag;
    }
}
exports.Starter = Starter;
